@@ -13,25 +13,23 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from __future__ import print_function
-
+import datetime
 import errno
 import functools
 import hashlib
 import json
 import os
 import re
-import six.moves.urllib.parse as urlparse
 import sys
 import threading
+import urllib.parse
 import uuid
 
-import six
 
-if os.name == 'nt':
-    import msvcrt
-else:
-    msvcrt = None
+if os.name == 'nt':  # noqa
+    import msvcrt  # noqa
+else:  # noqa
+    msvcrt = None  # noqa
 
 from oslo_utils import encodeutils
 from oslo_utils import strutils
@@ -161,7 +159,7 @@ def schema_args(schema_getter, omit=None):
                     # for the `join` to succeed. Enum types can also be `None`
                     # therefore, join's call would fail without the following
                     # list comprehension
-                    vals = [six.text_type(val) for val in property.get('enum')]
+                    vals = [str(val) for val in property.get('enum')]
                     description += ('Valid values: ' + ', '.join(vals))
                 kwargs['help'] = description
 
@@ -176,6 +174,75 @@ def pretty_choice_list(l):
     return ', '.join("'%s'" % i for i in l)
 
 
+def has_version(client, version):
+    versions = client.get('/versions')[1].get('versions')
+    supported = ['SUPPORTED', 'CURRENT', 'EXPERIMENTAL']
+    for version_struct in versions:
+        if version_struct['id'] == version:
+            return version_struct['status'] in supported
+    return False
+
+
+def print_cached_images(cached_images):
+    cache_pt = prettytable.PrettyTable(("ID",
+                                        "State",
+                                        "Last Accessed (UTC)",
+                                        "Last Modified (UTC)",
+                                        "Size",
+                                        "Hits"))
+    for item in cached_images:
+        state = "queued"
+        last_accessed = "N/A"
+        last_modified = "N/A"
+        size = "N/A"
+        hits = "N/A"
+        if item == 'cached_images':
+            state = "cached"
+            for image in cached_images[item]:
+                last_accessed = image['last_accessed']
+                if last_accessed == 0:
+                    last_accessed = "N/A"
+                else:
+                    last_accessed = datetime.datetime.utcfromtimestamp(
+                        last_accessed).isoformat()
+
+                cache_pt.add_row((image['image_id'], state,
+                                  last_accessed,
+                                  datetime.datetime.utcfromtimestamp(
+                                      image['last_modified']).isoformat(),
+                                  image['size'],
+                                  image['hits']))
+        else:
+            for image in cached_images[item]:
+                cache_pt.add_row((image,
+                                  state,
+                                  last_accessed,
+                                  last_modified,
+                                  size,
+                                  hits))
+
+    print(cache_pt.get_string())
+
+
+def print_dict_list(objects, fields):
+    pt = prettytable.PrettyTable([f for f in fields], caching=False)
+    pt.align = 'l'
+    for o in objects:
+        row = []
+        for field in fields:
+            field_name = field.lower().replace(' ', '_')
+            # NOTE (abhishekk) mapping field to actual name in the
+            # structure.
+            if field_name == 'task_id':
+                field_name = 'id'
+            data = o.get(field_name, '')
+            row.append(data)
+
+        pt.add_row(row)
+
+    print(encodeutils.safe_decode(pt.get_string()))
+
+
 def print_list(objs, fields, formatters=None, field_settings=None):
     '''Prints a list of objects.
 
@@ -216,8 +283,6 @@ def print_list(objs, fields, formatters=None, field_settings=None):
 
 def _encode(src):
     """remove extra 'u' in PY2."""
-    if six.PY2 and isinstance(src, unicode):
-        return src.encode('utf-8')
     return src
 
 
@@ -345,7 +410,7 @@ def get_file_size(file_obj):
     :retval: The file's size or None if it cannot be determined.
     """
     if (hasattr(file_obj, 'seek') and hasattr(file_obj, 'tell') and
-            (six.PY2 or six.PY3 and file_obj.seekable())):
+            file_obj.seekable()):
         try:
             curr = file_obj.tell()
             file_obj.seek(0, os.SEEK_END)
@@ -401,13 +466,13 @@ def strip_version(endpoint):
     # we make endpoint the first argument. However, we
     # can't do that just yet because we need to keep
     # backwards compatibility.
-    if not isinstance(endpoint, six.string_types):
+    if not isinstance(endpoint, str):
         raise ValueError("Expected endpoint")
 
     version = None
     # Get rid of trailing '/' if present
     endpoint = endpoint.rstrip('/')
-    url_parts = urlparse.urlparse(endpoint)
+    url_parts = urllib.parse.urlparse(endpoint)
     (scheme, netloc, path, __, __, __) = url_parts
     path = path.lstrip('/')
     # regex to match 'v1' or 'v2.0' etc
@@ -436,11 +501,18 @@ def integrity_iter(iter, checksum):
 
     :raises: IOError
     """
-    md5sum = hashlib.md5()
+    try:
+        md5sum = hashlib.new('md5')
+    except ValueError:
+        raise IOError(errno.EPIPE,
+                      'Corrupt image download. Expected checksum is %s '
+                      'but md5 algorithm is not available on the client' %
+                      checksum)
+
     for chunk in iter:
         yield chunk
-        if isinstance(chunk, six.string_types):
-            chunk = six.b(chunk)
+        if isinstance(chunk, str):
+            chunk = bytes(chunk, 'latin-1')
         md5sum.update(chunk)
     md5sum = md5sum.hexdigest()
     if md5sum != checksum:
@@ -449,6 +521,26 @@ def integrity_iter(iter, checksum):
                       (md5sum, checksum))
 
 
+def serious_integrity_iter(iter, hasher, hash_value):
+    """Check image data integrity using the Glance "multihash".
+
+    :param iter: iterable containing the image data
+    :param hasher: a hashlib object
+    :param hash_value: hexdigest of the image data
+    :raises: IOError if the hashdigest of the data is not hash_value
+    """
+    for chunk in iter:
+        yield chunk
+        if isinstance(chunk, str):
+            chunk = bytes(chunk, 'latin-1')
+        hasher.update(chunk)
+    computed = hasher.hexdigest()
+    if computed != hash_value:
+        raise IOError(errno.EPIPE,
+                      'Corrupt image download. Hash was %s expected %s' %
+                      (computed, hash_value))
+
+
 def memoized_property(fn):
     attr_name = '_lazy_once_' + fn.__name__
 
